# =======================================================================================================
# relevant R code used in the following publication:
# =======================================================================================================
# Title:   Environment and evolutionary history shape phylogenetic turnover in European tetrapods 
# Authors: Bianca Saladin, Wilfried Thuiller, Catherine H. Graham, Sebastien Lavergne, 
#          Luigi Maiorano, Nicolas Salamin, Niklaus E. Zimmermann
# Journal: Nature Communications
#-------------------------------------------------------------------------------------------------------#
# 18/11/2018 B.Saladin
#-------------------------------------------------------------------------------------------------------#
# Program: R (version 3.5.1)
#-------------------------------------------------------------------------------------------------------#
# Notes:   This is the code to run all analysis steps to generate the results for the main paper
#-------------------------------------------------------------------------------------------------------#

setwd("~/path to/dryad_data") 
# generate a file with the name "results" within the dryad_data folder - results can then be saved there.

require(betapart)
require(raster)
require(car)
require(lattice)
require(RColorBrewer)
require(rasterVis)
require(gdistance)
source("function/recstrat_prop.R") # function used for the stratified random sampling

# =======================================================================================================
# 1. Generating a stratified random sampling
# =======================================================================================================
# Input:   - distribution files at 15km resolution (based on Maiorano et al. 2013)
#          - climatic stratification layer with 9+9 classes (based on Frick and Hijmans 2017/Hijmans 2005)
# Output:  - sample points and their communities to calculate turnover in the next step
# Note:	   - this is a random sampling: each time executed, different communities are generated
#            therefore, the results will be different each time, and not exactly match those in the paper
#          - see Supplementary Fig. 2 for robustness of different sets of sampling sites used
# =======================================================================================================
# example for one taxon group ("amphibians" here):
spp     <- "amph"   #taxon group
bio     <- "bio9"   #bioclimatic variable for this taxon group used in combination with MIND
km      <- 15       #resolution (other resolutions were used for robustness analyses, see Supp.Inf.)
pts     <- 10       #maximum number of sampled sites within a stratum
classes <- "99"     #number of classes in the stratification layer (9+9 classes in both layers)
delta   <- "010"    #delta transformation of phylogenies ("010" means: no transformation applied)
author  <- "Roquet" #data set matched with the Roquet et al. 2014 phylogenies

# Random stratified sampling - Sampling of local communities proportional to the log of the stratum area
#-------------------------------------------------------------------------------------------------------#
in_grid   <- raster(paste("environment/stratificationLayers/",spp,"_",bio,"_MIND_strat.tif",sep=""))
prop_pts1 <- recstrat_prop(in_grid, pts) 
saveRDS(prop_pts1,file=paste("results/prop.points_classes",classes,"_",spp,"_",bio,"_",km,"km.rds",sep=""))
           
# get community data for all points
stk_spp <- stack(paste("distribution/",spp,"_",km,"km.gri",sep=""))
pts_spp <- extract(stk_spp,prop_pts1[,1:2]) 
rd_com  <- cbind(prop_pts1,pts_spp)
saveRDS(rd_com,file=paste("results/rd_com_classes",classes,"_",spp,"_",bio,"_",km,"km.rds",sep=""))
# =======================================================================================================

# =======================================================================================================
# 2. Calculating taxonomic and phylogenetic turnover 
# =======================================================================================================
# Input:   - sampled points and their communities ("rd_com")
#          - phylogenies (based on Roquet et al. 2014)
# Output:  - taxonomic turnover between all sampled points
#          - phylogenetic turnover between all sampled points (20x turnover, as 20 phylogenies are used)
# =======================================================================================================
# taxonomic turnover 
rd_com <- readRDS(paste("results/rd_com_classes",classes,"_",spp,"_",bio,"_",km,"km.rds",sep=""))
txBeta <- beta.pair(rd_com[,-c(1:3)]) 
save(txBeta,file=paste("results/taxB_classes",classes,"_",spp,"_",bio,"_",km,"km_",delta,"_rd_com",".RData",sep=""))        
# phylogenetic turnover (over 20 phylogenetic trees) 
load(paste("phylogenies/",author,"_",spp,"_",delta,".RData",sep="")) #loads "delta_R_50" object - only the first 20 trees are used
for (k in 1:20){
   phylo  <- delta_R_50[[k]]
   phBeta <- phylo.beta.pair(rd_com[,-c(1:3)],phylo) 
   save(phBeta,file=paste("results/phyB_classes",classes,"_",spp,"_",bio,"_",km,"km_",delta,"_phylo",k,".com",".RData",sep=""))     
}
# =======================================================================================================

# =======================================================================================================
# 3. Calculating least cost path distance -> geographic land distance
# =======================================================================================================
# Input:   - sample points
#          - environmental stratification layer
# Output:  - geographic land distances between sample pointsrd_com
# =======================================================================================================
cls <- raster(paste("environment/stratificationLayers/",spp,"_",bio,"_MIND_strat.tif",sep=""))
pts <- readRDS(paste("results/prop.points_classes",classes,"_",spp,"_",bio,"_",km,"km.rds",sep=""))

# generate conductance layer  (1/resistance) 
cd2 <- (1-(is.na(cls)*.99))   # min=0.01 (for sea), max=1.0 (for land)

coordinates(pts) <- c("x","y")
projection(pts)  <- "+proj=laea +lat_0=52 +lon_0=10 +x_0=4321000 +y_0=3210000 +ellps=GRS80 +units=m +no_defs"
trns <- transition(cd2,transitionFunction=mean, directions=8)
trns <- geoCorrection(trns, type="c",multpl=FALSE,scl=FALSE)
cost <- costDistance(trns,pts)
saveRDS(cost,file=paste("results/ldist_classes",classes,"_",spp,"_",bio,"_",km,"km.rds",sep=""))
# =======================================================================================================

# =======================================================================================================
# 4. Generating residuals of phylobeta ~ geographic land distance
# =======================================================================================================
# Input:   - phylogenetic turnover among sample points
#          - geographic land distances among sample points
# Output:  - dataframe with the residuals of (phylobeta ~ land distance) among neighboring strata
# =======================================================================================================
rd_com <- readRDS(paste("results/rd_com_classes",classes,"_",spp,"_",bio,"_",km,"km.rds",sep=""))
xy.coord <- rd_com[,1:2]
d.geo <- readRDS(paste("results/ldist_classes",classes,"_",spp,"_",bio,"_",km,"km.rds",sep=""))

layerID   <- data.frame(cbind(as.numeric(rd_com$strata_id),as.numeric(row.names(rd_com))))
names(layerID) <- c("layer","ID")

for (o in 1:20){
   load(file=paste("results/phyB_classes",classes,"_",spp,"_",bio,"_",km,"km_010_phylo",o,".com",".RData",sep=""))
   d.tur <- car::logit(phBeta$phylo.beta.sim,adjust=0.01)
#  lm.tmp <- lm(d.tur~d.geo)
#  resid  <- as.matrix(lm.tmp$resid)
#  or quadratic term
   lm.tmp<- lm(d.tur~d.geo+I(d.geo^2))
   resid <- as.matrix(lm.tmp$resid)
   df    <- data.frame(matrix(data=NA,ncol=3))   
   names(df) <- c("icoord","jcoord","mean")

   #fill the df horizontal
   isteps <- seq(from=1,to=8,by=1)
   jsteps <- seq(from=1,to=9,by=1)
   k <- 1
   for (j in jsteps){
      for (i in isteps){
         df[k,1] <- (i+i+1)/2
         df[k,2] <- j
         l <- as.numeric(j*10+i)
         l1 <- which(layerID$layer%in%l) 
         r1 <- which(layerID$layer%in%as.numeric(l+1))
            if(!length(l1)){ 
      	       df[k,3] <- "NaN" 
               }  else { 
                     df[k,3] <- mean(resid[l1,r1],na.rm=T)
                     }
         k <- k+1
      }
   }
   df[df=="-Inf"] <- "Na"
   df[df=="Inf"]  <- "Na"
   #fill the df vertical
   isteps <- seq(from=1,to=9,by=1)
   jsteps <- seq(from=1,to=8,by=1)
   for ( i in isteps){
      for (j in jsteps){
         df[k,2] <- (j+j+1)/2
         df[k,1] <- i
         l  <- as.numeric(j*10+i)
         l1 <- which(layerID$layer%in%l) 
         n1 <- which(layerID$layer%in%as.numeric(l+10))
            if(!length(l1)){ 
               df[k,3] <- "NaN" 
               }  else { 
                     df[k,3] <- mean(resid[l1, n1],na.rm=T)
                     }
               k <- k+1
         }
      }
      df[df=="-Inf"] <- "Na"
      df[df=="Inf"]  <- "Na"
   for (b in 1:3){df[,b] <- as.numeric(df[,b])}
   save(df,file=paste("results/ldist_classes",classes,"_",spp,".df_010_com_phylo",o,"_",km,"km.RData",sep=""))
}
# =======================================================================================================

# =======================================================================================================
# 5. Interpolation to environmental space
# =======================================================================================================
# Input:   - dataframe with the residuals of lm(phylobeta ~ land distance)
#          - environmental layers (temperature and moisture index)
# Output:  - interpolated and rasterized phylobeta (residuals) in environmental space
# =======================================================================================================
load(paste("results/ldist_classes",classes,"_",spp,".df_010_com_phylo",1,"_",km,"km.RData",sep="")) #loads "df" object
ldf <- dim(df)[1]
mean.20_df <- data.frame(matrix(data=NA,ncol=23,nrow=ldf))
names(mean.20_df) <- c("icoord","jcoord",rep("phy",20),"mean.all")
# mean phyloBeta over the 20 phylogenies
for (o in 1:20){
   load(paste("results/ldist_classes",classes,"_",spp,".df_010_com_phylo",o,"_",km,"km.RData",sep=""))
   mean.20_df$icoord <- df$icoord
   mean.20_df$jcoord <- df$jcoord
   mean.20_df[,o]    <- df$mean
}
mean.20_df[,23] <- apply(mean.20_df[,c(3:22)],1,mean,na.rm=T)

#  Rasterize
data <- mean.20_df  #shorter name
x <- data$icoord
y <- data$jcoord
z <- data$mean.all
xyz <- cbind(x,y,z) 
rast <- raster(ncol=17,nrow=17) 
ext <- c(0.75,9.25,0.75,9.25) 
extent(rast) <- ext
ttt <- rasterize(xyz[,c(1:2)],rast,xyz[,3],fun=mean)
crs(ttt)<-""
rast1 <- raster(ncol=10,nrow=10)
ext <- c(0.75,9.25,0.75,9.25)
extent(rast1) <- ext
tt1 <- resample(ttt,rast1,method="bilinear",expand=T)
crs(tt1) <- ""
rast1b <- raster(ncol=20,nrow=20) 
ext <-c(0.75,9.25,0.75,9.25) 
extent(rast1b)<-ext
tt1b<-resample(tt1,rast1b,method="bilinear",expand=T)
crs(tt1b)<-""
rast2 <- raster(ncol=100,nrow=100)
ext <- c(0.75,9.25,0.75,9.25) 
extent(rast2) <- ext
tt2 <- resample(tt1b,rast2,method="bilinear")
crs(tt2) <- ""
tt3a <- resample(tt1,tt2,method="ngb")
int_phyB <- mask(tt2,tt3a)

# for the levelplot
col1.c <- colorRampPalette(c("#F5F5DC","LemonChiffon","#FDD017","LightSeaGreen","#368BC1","#2B547E","#810541")) 
ival <- ((int_phyB@extent@xmax-int_phyB@extent@xmin)/5)
tcks <- 0.5+(seq(1,5)*ival) 
bioMIND_masked <- raster(paste("environment/mnMIND678.tif",sep=""))
bioTEMP_masked <- raster(paste("environment/",bio,".tif",sep=""))
mn.temp    <- minValue(bioTEMP_masked)/10 
mx.temp    <- maxValue(bioTEMP_masked)/10
seq.temp   <- mx.temp-mn.temp
by.temp    <- seq.temp/5
grad.temp  <- seq(from=mn.temp,to=mx.temp,by=by.temp)
mn.MIND    <- minValue(bioMIND_masked)
mx.MIND    <- maxValue(bioMIND_masked)
seq.MIND   <- mx.MIND-mn.MIND
by.MIND    <- seq.MIND/5
grad.MIND  <- seq(from=mn.MIND,to=mx.MIND,by=by.MIND)
  
file.mn <- paste("results/phyloB_Resid_classes",classes,"_",spp,".df.010_com_",km,"_mn.pdf",sep="")
pdf(file=file.mn)
   print(levelplot(int_phyB,margin=F, colorkey = list(space='right',labels = list(cex=1.5,font=1.5,col="black"), height=0.65, width=2),
                   scales=list(x=list(at=tcks,cex=1.5,labels=c(round(grad.temp[2:10],1))),
                               y=list(at=tcks,cex=1.5,labels=c(round(grad.MIND[2:10],1)))),
                               ylab=list(label="moisture index",cex=2),
                               xlab=list(label="temperature",cex=2),
                               contour=F,at= unique(seq(minValue(int_phyB),maxValue(int_phyB),length=100)),col.regions = col1.c(100))
                    )
                    trellis.focus("legend",side="right", clipp.off=TRUE, highlight=FALSE)
dev.off()
# =======================================================================================================
# End of code
# =======================================================================================================





